@@ -70,18 +70,13 @@
 
 #include "nsContentUtils.h"
 
-// Statics
-imgILoader* nsImageLoadingContent::sImgLoader = nsnull;
-nsIIOService* nsImageLoadingContent::sIOService = nsnull;
-
-
 nsImageLoadingContent::nsImageLoadingContent()
   : mObserverList(nsnull),
     mLoadingEnabled(PR_TRUE),
     mImageIsBlocked(PR_FALSE),
     mHaveHadObserver(PR_FALSE)
 {
-  if (!sImgLoader)
+  if (!nsContentUtils::GetImgLoader())
     mLoadingEnabled = PR_FALSE;
 }
 
@@ -98,23 +93,6 @@ nsImageLoadingContent::~nsImageLoadingContent()
                "Observers still registered?");
 }
 
-void
-nsImageLoadingContent::Initialize()
-{
-  // If this fails, NS_NewURI will try to get the service itself
-  CallGetService("@mozilla.org/network/io-service;1", &sIOService);
-
-  // Ignore failure and just don't load images
-  CallGetService("@mozilla.org/image/loader;1", &sImgLoader);
-}
-
-void
-nsImageLoadingContent::Shutdown()
-{
-  NS_IF_RELEASE(sImgLoader);
-  NS_IF_RELEASE(sIOService);
-}
-
 // Macro to call some func on each observer.  This handles observers
 // removing themselves.
 #define LOOP_OVER_OBSERVERS(func_)                                       \
@@ -230,7 +208,7 @@ nsImageLoadingContent::GetLoadingEnabled(PRBool *aLoadingEnabled)
 NS_IMETHODIMP
 nsImageLoadingContent::SetLoadingEnabled(PRBool aLoadingEnabled)
 {
-  if (sImgLoader)
+  if (nsContentUtils::GetImgLoader())
     mLoadingEnabled = aLoadingEnabled;
   return NS_OK;
 }
@@ -356,7 +334,7 @@ nsImageLoadingContent::LoadImageWithChannel(nsIChannel* aChannel,
   
   NS_ENSURE_ARG_POINTER(aChannel);
 
-  if (!sImgLoader)
+  if (!nsContentUtils::GetImgLoader())
     return NS_ERROR_NULL_POINTER;
 
   // XXX what should we do with content policies here, if anything?
@@ -372,7 +350,7 @@ nsImageLoadingContent::LoadImageWithChannel(nsIChannel* aChannel,
 
   nsCOMPtr<imgIRequest> & req = mCurrentRequest ? mPendingRequest : mCurrentRequest;
 
-  return sImgLoader->LoadImageWithChannel(aChannel, this, doc, aListener, getter_AddRefs(req));
+  return nsContentUtils::GetImgLoader()->LoadImageWithChannel(aChannel, this, doc, aListener, getter_AddRefs(req));
 }
 
 // XXX This should be a protected method, not an interface method!!!
@@ -550,7 +528,7 @@ nsImageLoadingContent::StringToURI(const nsACString& aSpec,
                    aSpec,
                    charset.IsEmpty() ? nsnull : PromiseFlatCString(charset).get(),
                    baseURL,
-                   sIOService);
+                   nsContentUtils::GetIOServiceWeakRef());
 }
 
 
@@ -597,7 +575,7 @@ HandleImagePLEvent(PLEvent* aEvent)
     eventMsg = NS_IMAGE_ERROR;
   }
 
-  nsEvent event(eventMsg);
+  nsEvent event(PR_TRUE, eventMsg);
   evt->mContent->HandleDOMEvent(evt->mPresContext, &event, nsnull,
                                 NS_EVENT_FLAG_INIT, &estatus);
 
