@@ -44,6 +44,9 @@
 #include "nsString.h"
 #include "nsIDOMInstallVersion.h"
 #include "nsIDOMInstallTriggerGlobal.h"
+#include "nsIDOMWindow.h"
+#include "nsIDOMDocument.h"
+#include "nsIDocument.h"
 #include "nsIDocShell.h"
 #include "nsIObserverService.h"
 #include "nsInstallTrigger.h"
@@ -67,11 +70,25 @@ extern PRBool ConvertJSValToBool(PRBool* aProp,
                                 JSContext* aContext,
                                 jsval aValue);
 
-extern PRBool ConvertJSValToObj(nsISupports** aSupports,
-                               REFNSIID aIID,
-                               const nsString& aTypeName,
-                               JSContext* aContext,
-                               jsval aValue);
+PR_STATIC_CALLBACK(void)
+FinalizeInstallTriggerGlobal(JSContext *cx, JSObject *obj);
+
+/***********************************************************************/
+//
+// class for InstallTriggerGlobal
+//
+JSClass InstallTriggerGlobalClass = {
+  "InstallTrigger",
+  JSCLASS_HAS_PRIVATE,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_EnumerateStub,
+  JS_ResolveStub,
+  JS_ConvertStub,
+  FinalizeInstallTriggerGlobal
+};
 
 //
 // InstallTriggerGlobal finalizer
@@ -129,13 +146,69 @@ static JSBool CreateNativeObject(JSContext *cx, JSObject *obj, nsIDOMInstallTrig
     return JS_TRUE;
 }
 
+//
+// Helper function for URI verification
+//
+static nsresult
+InstallTriggerCheckLoadURIFromScript(JSContext *cx, const nsAString& uriStr)
+{
+    nsresult rv;
+    nsCOMPtr<nsIScriptSecurityManager> secman(
+        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID,&rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // get the script base URI
+    nsCOMPtr<nsIURI> scriptURI;
+    nsCOMPtr<nsIPrincipal> principal;
+    rv = secman->GetSubjectPrincipal(getter_AddRefs(principal));
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (!principal)
+        return NS_ERROR_FAILURE;
+
+    rv = principal->GetURI(getter_AddRefs(scriptURI));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (!scriptURI) {
+      // No URI reachable from the principal, get one from the calling
+      // window.
+
+      nsIScriptContext *scx = GetScriptContextFromJSContext(cx);
+      NS_ENSURE_TRUE(scx, NS_ERROR_FAILURE);
+
+      nsCOMPtr<nsIDOMWindow> window =
+        do_QueryInterface(scx->GetGlobalObject());
+      NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
+
+      nsCOMPtr<nsIDOMDocument> domDoc;
+      window->GetDocument(getter_AddRefs(domDoc));
+
+      nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
+      NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
+
+      scriptURI = doc->GetDocumentURI();
+    }
+
+    // convert the requested URL string to a URI
+    nsCOMPtr<nsIURI> uri;
+    rv = NS_NewURI(getter_AddRefs(uri), uriStr);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // are we allowed to load this one?
+    rv = secman->CheckLoadURI(scriptURI, uri,
+                    nsIScriptSecurityManager::DISALLOW_SCRIPT_OR_DATA);
+    return rv;
+}
+
 //
 // Native method UpdateEnabled
 //
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalUpdateEnabled(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
 
   *rval = JSVAL_FALSE;
 
@@ -160,8 +233,11 @@ InstallTriggerGlobalUpdateEnabled(JSContext *cx, JSObject *obj, uintN argc, jsva
 //
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
-{
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+{ 
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
 
   *rval = JSVAL_FALSE;
 
@@ -188,6 +264,21 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
   }
 
 
+  nsCOMPtr<nsIScriptSecurityManager> secman(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
+  if (!secman)
+  {
+    JS_ReportError(cx, "Could not the script security manager service.");
+    return JS_FALSE;
+  }
+  // get the principal.  if it doesn't exist, die.
+  nsCOMPtr<nsIPrincipal> principal;
+  secman->GetSubjectPrincipal(getter_AddRefs(principal));
+  if (!principal)
+  {
+    JS_ReportError(cx, "Could not get the Subject Principal during InstallTrigger.Install()");
+    return JS_FALSE;
+  }
+  
   // get window.location to construct relative URLs
   nsCOMPtr<nsIURI> baseURL;
   JSObject* global = JS_GetGlobalObject(cx);
@@ -202,11 +293,7 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
     }
   }
 
-  // if we can't create a security manager we might be in the wizard, allow
   PRBool abortLoad = PR_FALSE;
-  nsCOMPtr<nsIScriptSecurityManager> secman(
-      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-
 
   // parse associative array of installs
   if ( argc >= 1 && JSVAL_IS_OBJECT(argv[0]) )
@@ -215,6 +302,8 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
     if (!trigger)
       return JS_FALSE;
 
+    trigger->SetPrincipal(principal);
+
     JSIdArray *ida = JS_Enumerate( cx, JSVAL_TO_OBJECT(argv[0]) );
     if ( ida )
     {
@@ -254,19 +343,6 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
                 xpiURL = NS_ConvertUTF8toUTF16(resolvedURL);
             }
 
-            // Make sure we're allowed to load this URL
-            if (secman)
-            {
-                nsCOMPtr<nsIURI> uri;
-                nsresult rv = NS_NewURI(getter_AddRefs(uri), xpiURL);
-                if (NS_SUCCEEDED(rv))
-                {
-                    rv = secman->CheckLoadURIFromScript(cx, uri);
-                    if (NS_FAILED(rv))
-                        abortLoad = PR_TRUE;
-                }
-            }
-
             nsAutoString icon(iconURL);
             if (iconURL && baseURL)
             {
@@ -275,13 +351,28 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
                 icon = NS_ConvertUTF8toUTF16(resolvedIcon);
             }
 
-            nsXPITriggerItem *item = new nsXPITriggerItem( name, xpiURL.get(), icon.get() );
-            if ( item )
-            {
-                trigger->Add( item );
-            }
-            else
+            // Make sure we're allowed to load this URL and the icon URL
+            nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, xpiURL);
+            if (NS_FAILED(rv))
                 abortLoad = PR_TRUE;
+
+            if (!abortLoad && iconURL)
+            {
+                rv = InstallTriggerCheckLoadURIFromScript(cx, icon);
+                if (NS_FAILED(rv))
+                    abortLoad = PR_TRUE;
+            }
+
+            if (!abortLoad)
+            {
+                nsXPITriggerItem *item = new nsXPITriggerItem( name, xpiURL.get(), icon.get() );
+                if ( item )
+                {
+                    trigger->Add( item );
+                }
+                else
+                    abortLoad = PR_TRUE;
+            }
         }
         else
             abortLoad = PR_TRUE;
@@ -320,7 +411,11 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   uint32       chromeType = NOT_CHROME;
   nsAutoString sourceURL;
   nsAutoString name;
@@ -383,20 +478,9 @@ InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsva
     }
 
     // Make sure caller is allowed to load this url.
-    // if we can't create a security manager we might be in the wizard, allow
-    nsCOMPtr<nsIScriptSecurityManager> secman(
-        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-    if (secman)
-    {
-        nsCOMPtr<nsIURI> uri;
-        nsresult rv = NS_NewURI(getter_AddRefs(uri), sourceURL);
-        if (NS_SUCCEEDED(rv))
-        {
-            rv = secman->CheckLoadURIFromScript(cx, uri);
-            if (NS_FAILED(rv))
-                return JS_FALSE;
-        }
-    }
+    nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, sourceURL);
+    if (NS_FAILED(rv))
+        return JS_FALSE;
 
     if ( chromeType & CHROME_ALL )
     {
@@ -420,7 +504,11 @@ InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsva
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRBool       nativeRet;
   PRInt32      flags = 0;
 
@@ -475,20 +563,9 @@ InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc
     }
 
     // Make sure caller is allowed to load this url.
-    // if we can't create a security manager we might be in the wizard, allow
-    nsCOMPtr<nsIScriptSecurityManager> secman(
-        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-    if (secman)
-    {
-        nsCOMPtr<nsIURI> uri;
-        nsresult rv = NS_NewURI(getter_AddRefs(uri), xpiURL);
-        if (NS_SUCCEEDED(rv))
-        {
-            rv = secman->CheckLoadURIFromScript(cx, uri);
-            if (NS_FAILED(rv))
-                return JS_FALSE;
-        }
-    }
+    nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, xpiURL);
+    if (NS_FAILED(rv))
+        return JS_FALSE;
 
     if (argc >= 2 && !JS_ValueToInt32(cx, argv[1], (int32 *)&flags))
     {
@@ -517,7 +594,11 @@ InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalCompareVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString regname;
   nsAutoString version;
   int32        major,minor,release,build;
@@ -616,7 +697,10 @@ InstallTriggerGlobalCompareVersion(JSContext *cx, JSObject *obj, uintN argc, jsv
 PR_STATIC_CALLBACK(JSBool)
 InstallTriggerGlobalGetVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)
+    JS_GetInstancePrivate(cx, obj, &InstallTriggerGlobalClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
 
   nsAutoString regname;
   nsAutoString version;
@@ -651,23 +735,6 @@ InstallTriggerGlobalGetVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *
   return JS_TRUE;
 }
 
-/***********************************************************************/
-//
-// class for InstallTriggerGlobal
-//
-JSClass InstallTriggerGlobalClass = {
-  "InstallTrigger",
-  JSCLASS_HAS_PRIVATE,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_EnumerateStub,
-  JS_ResolveStub,
-  JS_ConvertStub,
-  FinalizeInstallTriggerGlobal
-};
-
 //
 // InstallTriggerGlobal class methods
 //
